library TextLine;
{$IFDEF WIN64}
  {$E w64}
  {$IFNDEF VER230}
    {$E wdx64}
  {$ENDIF}
{$ENDIF}
{$IFDEF WIN32}
  {$E wdx}
{$ENDIF}
{$WEAKLINKRTTI ON}

{$IFNDEF VER230}
   Delphi XE2
{$ENDIF}

uses
  Windows,
  SysUtils,
  Classes,
  ContPlug, SProc,
  WideStrUtils;

const
  defReplaces = 50;
  defFields = 30;
  highReplaces = 1000;
  highFields = 200;
  lowFields = 10;
  lowReplaces = 0;


var
  fnIni: string;
  fExt: string;
  fSkip: boolean;
  fReplaces: array of record sFrom, sTo: string;
end;
  fNeedReplace: Boolean;
  MaxReplaces: Integer = defReplaces;
  MaxFields: Integer = defFields;

// --------------------------------------------
function ContentGetSupportedField(FieldIndex: Integer;
  FieldName, Units: pAnsiChar; maxlen: Integer): Integer; stdcall;
begin
  if (FieldIndex < 0) or (FieldIndex > (MaxFields - 1)) then
  begin
    Result := FT_NOMOREFIELDS;
    Exit
  end;

  StrLCopy(FieldName, pAnsiChar(Ansistring(IntToStr(FieldIndex + 1))), maxlen - 1);
  StrLCopy(Units, 'win|dos|koi8-r|mac', maxlen - 1);
  Result := FT_STRINGW;
end;

// ------------
procedure Init;
var
  n, Counter, CurIndex: Integer;
  s, fnEx: string;
  ReplaceFlag: Boolean;
begin
  fNeedReplace := True;
  ReplaceFlag := False;

  fnIni := ChangeFileExt(GetModuleName(HInstance), '.ini');
  fnEx := ChangeFileExt(GetModuleName(HInstance), '.example.ini');
  if FileExists(fnEx) and not FileExists(fnIni) then
    CopyFile(PChar(fnEx), PChar(fnIni), true);

  fExt := GetIniKey('Options', 'Extensions', '', fnIni);
  if fExt <> '' then
    fExt := ' ' + fExt + ' ';
  fSkip := (GetIniKey('Options', 'SkipEmpty', '0', fnIni) <> '0');

  MaxReplaces := GetIniKeyInteger('Options', 'MaxReplaces', fnIni, defReplaces);
  if MaxReplaces < lowReplaces then
    MaxReplaces := lowReplaces
  else if MaxReplaces > highReplaces then
    MaxReplaces := highReplaces;

  MaxFields := GetIniKeyInteger('Options', 'MaxFields', fnIni, defFields);
  if MaxFields < lowFields then
    MaxFields := lowFields
  else if MaxFields > highFields then
    MaxFields := highFields;

  CurIndex := Low(fReplaces);
  SetLength(fReplaces, MaxReplaces);

  for n := Low(fReplaces) to High(fReplaces) do
  begin
    s := GetIniKey('Replaces', 'S' + IntToStr(n + 1), '=', fnIni);
    fReplaces[CurIndex].sFrom := Copy(s, 1, Pos('=', s) - 1);
    fReplaces[CurIndex].sTo := Copy(s, Pos('=', s) + 1, MaxInt);
    if not ReplaceFlag then
      ReplaceFlag := fReplaces[CurIndex].sFrom <> '';
    if fReplaces[CurIndex].sFrom <> '' then
      Inc(CurIndex);
  end;
  fNeedReplace := ReplaceFlag;
  SetLength(fReplaces, CurIndex);
end;

// --------------------------------------------
function ContentGetValue(fn: pAnsiChar; FieldIndex, UnitIndex: integer;
  FieldValue: pAnsiChar; maxlen, Flags: integer): integer; stdcall;
begin
  Result := ft_fieldempty;
end;

function ContentGetValueW(FileName: pWideChar; FieldIndex, UnitIndex: integer;
  FieldValue: pWideChar; maxlen, Flags: integer): integer; stdcall;
var
  s, ext: string;
  n, EmptyNum: integer;
  TR: TStreamReader;
  fn: string;
begin
  if (FieldIndex < 0) or (FieldIndex > (MaxFields - 1)) then
  begin
    Result := FT_NOSUCHFIELD;
    Exit
  end;

  fn := FileName;
  if Length(fn) > 255 then
    if Pos('\\', fn) > 0 then
      fn := '\\?\UNC\' + Copy(fn, 3, Length(fn) - 2)
    else
      fn := '\\?\' + fn;

  ext := LowerCase(ExtractFileExt(fn));
  if (ext = '') then
    ext := '.';
  if (ext <> '') and (Pos('.', ext) = 1) then
    Delete(ext, 1, 1);
  if (fExt <> '') and (Pos(' ' + ext + ' ', fExt) = 0) then
  begin
    Result := FT_FILEERROR;
    Exit
  end;

  // if (Flags and CONTENT_DELAYIFSLOW)>0 then
  // begin Result:= FT_DELAYED; Exit end;
  Result := ft_fieldempty;

  try
    TR := TStreamReader.Create(fn, TEncoding.ANSI, true);
    try
      n := 1;
      EmptyNum := 0;
      repeat
        s := '';
        if TR.EndOfStream then
          Break;
        s := TR.ReadLine;
        s := Trim(s);

        if fSkip and (s ='') then
          Continue;

        inc(n);
        // if (s='') and fSkip then inc(EmptyNum);    TO DO !
      until n = FieldIndex + 2 + EmptyNum;
    finally
      TR.Free;
    end;
  except
    s := '';
  end;

  if s = '' then
    Exit;

  if IsUTF8String(s) then
    s := UTF8ToUnicodeString(s);

  if UnitIndex = 2 then
    s := SCodepageToUnicodeAPI(s, 20866);
  if UnitIndex = 3 then
    s := SCodepageToUnicodeAPI(s, CP_MACCP);

  if fNeedReplace then
  begin
    for n := low(fReplaces) to high(fReplaces) do
      with fReplaces[n] do
        if sFrom <> '' then
          SReplaceAll(s, sFrom, sTo);
  end;

  if UnitIndex = 1 then
    s := ToANSI(s);

  if s <> '' then
  begin
    StrLCopy(FieldValue, pWideChar(WideString(s)), maxlen div 2 - 2);
    Result := FT_STRINGW;
  end;
end;

exports
  ContentGetSupportedField,
  ContentGetValue,
  ContentGetValueW;

procedure IntDllProc(Code: integer);
begin
  case Code of
  DLL_PROCESS_ATTACH: begin
    Init;
  end;
  DLL_PROCESS_DETACH: begin
    fReplaces := nil;
  end;
  end; //case
end;

begin
  DllProc := @IntDllProc;
  IntDllProc(DLL_PROCESS_ATTACH);
end.
